#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import cvxpy as cp
import numpy as np

# Generate a random symmetric matrix with 1s in the diagonal.
n = 5
X0 = np.random.normal(0,1, (n,n))
X0 = (X0 + X0.T)/2
for i in range(n):
    X0[i][i] = 1

X = cp.Variable((n,n), symmetric=True)
# X >> 0 indicates that X must be positive semidefinite
constraints = [X >> 0]
constraints += [
    X[i][i] == 1 for i in range(n)
]
sigma = 0.1
constraints += [X  - sigma*np.identity(n)>> 0 ]

prob = cp.Problem(cp.Minimize(cp.norm(X-X0,"fro")),
                  constraints)

prob.solve(verbose=True)

# Print result.
print("X0 = ", X0)
print("Eigenvalues of X0 = ", np.linalg.eigvals(X0))
print()
print("The optimal value is", prob.value)
print("The solution X is")
print(X.value)
print("Eigenvalues of X = ", np.linalg.eigvals(X.value))